//+------------------------------------------------------------------+
//|                                                    Percenter.mq4 |
//|                                   Copyright  2007, Serega Lykov |
//|                                       http://mtexperts.narod.ru/ |
//+------------------------------------------------------------------+

#property copyright "Copyright  2007, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

//---- property of indicator ----------------------------------------+
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Gray
#property indicator_color2 Green
#property indicator_color3 Red
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2

//---- external parameters ------------------------------------------+
extern int    Period_Ind      = 20;  // the period of MA
extern int    Method_Ind      = 0;   // the method of MA: 0-Simple, 1-Exponential, 2-Smoothed, 3-Weighted
extern int    Signal_Period   = 20;  // the period of signal MA
extern int    Signal_Method   = 0;   // the method of signal MA: 0-Simple, 1-Exponential, 2-Smoothed, 3-Weighted
extern int    Signal_Price    = 0;   // the price for signal MA: 0-Close, 1-Open, 2-Typical, 3-Weighted
extern bool   UseSmooth       = true;// true - use smoothing of value
extern int    MA_Period       = 7;   // the period of smoothing MA
extern int    MA_Method       = 0;   // the method of smoothing MA: 0-Simple, 1-Exponential, 2-Smoothed, 3-Weighted

//---- buffers ------------------------------------------------------+
static double SkillBuffer[];
static double SmoothBuffer[];
static double HistogrammaUp[];
static double HistogrammaDown[];

//---- global variables ---------------------------------------------+
static int    sig_price;
static int    draw_begin;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- the 1 additional buffer
   IndicatorBuffers(4);
//---- set a "short" name of the indicator
   if(Method_Ind == 3) string ind_ma = "LWMA(";
   else if(Method_Ind == 2) ind_ma = "SMMA(";
        else if(Method_Ind == 1) ind_ma = "EMA(";
             else ind_ma = "SMA(";
   if(Signal_Method == 3) string sig_ma = "LWMA(";
   else if(Signal_Method == 2) sig_ma = "SMMA(";
        else if(Signal_Method == 1) sig_ma = "EMA(";
             else sig_ma = "SMA(";
   if(UseSmooth == false) string smt_ma = "";
   else
     {
      if(MA_Method == 3) smt_ma = "LWMA(";
      else if(MA_Method == 2) smt_ma = "SMMA(";
           else if(MA_Method == 1) smt_ma = "EMA(";
                else smt_ma = "SMA(";
      smt_ma = " with Smooth " + smt_ma + MA_Period + ")";
     }
   IndicatorShortName("Percenter "+ind_ma+Period_Ind+"),"+sig_ma+Signal_Period+")"+smt_ma);
//---- set a accuracy of values of the indicator
   IndicatorDigits(2);
//---- set a horizontal levels
   SetLevelValue(0,00);
   SetLevelValue(1,30);
   SetLevelValue(2,-30);
//---- set a style for lines
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID,2);
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID,2);
   SetIndexStyle(2,DRAW_HISTOGRAM,STYLE_SOLID,2);
//---- set a arrays for lines
   if(UseSmooth == false)
     {
      SetIndexBuffer(0,SkillBuffer);
      SetIndexBuffer(1,HistogrammaUp);
      SetIndexBuffer(2,HistogrammaDown);
      SetIndexBuffer(3,SmoothBuffer);
     }
   else
     {
      SetIndexBuffer(0,SmoothBuffer);
      SetIndexBuffer(1,HistogrammaUp);
      SetIndexBuffer(2,HistogrammaDown);
      SetIndexBuffer(3,SkillBuffer);
     }
//---- set a first bar for drawing the lines
   draw_begin = MathMax(Period_Ind,Signal_Period) + MA_Period;
   SetIndexDrawBegin(0,draw_begin);
   SetIndexDrawBegin(1,draw_begin);
   SetIndexDrawBegin(2,draw_begin);
//---- set a names for lines
   SetIndexLabel(0,"Percenter");
   SetIndexLabel(1,"PercenterUp");
   SetIndexLabel(2,"PercenterDown");
//---- set a constant
   if(Signal_Price == 3) sig_price = PRICE_WEIGHTED;
   else if(Signal_Price == 2) sig_price = PRICE_TYPICAL;
        else if(Signal_Price == 1) sig_price = PRICE_OPEN;
             else sig_price = PRICE_CLOSE;
//---- finish of initialization
   return(0);
  }

//+------------------------------------------------------------------+
//| Simple oscillator                                                |
//+------------------------------------------------------------------+
int start()
  {
   // if on the chart there are not enough bars the indicator is not calculate
   if(Bars <= MathMax(Period_Ind,Signal_Period)) return(0);
   // amount not changed bars after last call of the indicator 
   int counted_bars = IndicatorCounted();
   if(counted_bars < 0) return(-1);
   // last counted bar will be counted
   if(counted_bars > 0) counted_bars--;
   int limit = Bars - counted_bars;
   if(limit > Bars - draw_begin) limit = Bars - draw_begin;
   for(int i=0; i<limit; i++)
     {
      double UpRound   = iMA(NULL,0,Period_Ind,0,Method_Ind,PRICE_HIGH,i);
      double DownRound = iMA(NULL,0,Period_Ind,0,Method_Ind,PRICE_LOW,i);
      double SignalLine = iMA(NULL,0,Signal_Period,0,Signal_Method,sig_price,i);
      if(UpRound == DownRound) UpRound++;
      SkillBuffer[i] = ((SignalLine - DownRound)*100) / (UpRound - DownRound) - 50;
     }
   if(UseSmooth != false)
     {
      for(i=0; i<limit; i++) SmoothBuffer[i] = iMAOnArray(SkillBuffer,0,MA_Period,0,MA_Method,i);
      DrawColorHist(SmoothBuffer,limit);
     }
   else
     {
      for(i=0; i<limit; i++) SmoothBuffer[i] = EMPTY_VALUE;
      DrawColorHist(SkillBuffer,limit);
     }
   return(0);
  }
  
void DrawColorHist(double &array[], int limit)
  {
   for(int i=0; i<limit; i++)
     {
      if(array[i] > array[i+1])
        {
         HistogrammaUp[i]   = array[i];
         HistogrammaDown[i] = EMPTY_VALUE;
        }
      else
        {
         if(array[i] < array[i+1])
             {
              HistogrammaDown[i] = array[i];
              HistogrammaUp[i]   = EMPTY_VALUE;
             }
         else
           {
            if(HistogrammaDown[i+1] != EMPTY_VALUE)
              {
               HistogrammaUp[i]   = array[i];
               HistogrammaDown[i] = EMPTY_VALUE;
              }
            else
              {
               HistogrammaDown[i] = array[i];
               HistogrammaUp[i]   = EMPTY_VALUE;
              }
           }
        }
     }
  }

//+------------------------------------------------------------------+